package app.analysis.sentiment;

import edu.stanford.nlp.ling.CoreAnnotations;
import edu.stanford.nlp.neural.rnn.RNNCoreAnnotations;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.sentiment.SentimentCoreAnnotations;
import edu.stanford.nlp.trees.Tree;
import org.ejml.simple.SimpleMatrix;

import java.util.*;

public class SentimentAnalyzer {

    private StanfordCoreNLP pipeline;

    public SentimentAnalyzer() {
        this.initialise();
    }

    private void initialise() {
        Properties properties = new Properties();
        properties.setProperty("annotators", "tokenize, ssplit, parse, sentiment");
        pipeline = new StanfordCoreNLP(properties);
    }

    public SentimentResult analyse(String input) {
        SentimentClassification classification = new SentimentClassification();
        SentimentResult result = new SentimentResult();

        if(input != null && input.length() > 0) {
            Annotation annotation = pipeline.process(input);
            annotation.get(CoreAnnotations.SentencesAnnotation.class).forEach(sentence -> {
                Tree tree = sentence.get(SentimentCoreAnnotations.SentimentAnnotatedTree.class);
                SimpleMatrix simpleMatrix  = RNNCoreAnnotations.getPredictions(tree);

                classification.setVeryNegative((int)Math.round(simpleMatrix.get(0)*100d));
                classification.setNegative((int)Math.round(simpleMatrix.get(1)*100d));
                classification.setNeutral((int)Math.round(simpleMatrix.get(2)*100d));
                classification.setPositive((int)Math.round(simpleMatrix.get(3)*100d));
                classification.setVeryPositive((int)Math.round(simpleMatrix.get(4)*100d));

                String type = sentence.get(SentimentCoreAnnotations.SentimentClass.class);
                result.setType(type);
                result.setClassification(classification);
                result.setScore(RNNCoreAnnotations.getPredictedClass(tree));
            });
            List<String> words = new ArrayList<>();
            annotation.get(CoreAnnotations.TokensAnnotation.class).forEach(word -> {
                if(!word.word().matches("(?![@',&])\\p{Punct}")) {
                    words.add(word.word());
                }
            });
            result.setText(words);
        }
        return result;
    }
}
