package app.services;

import app.entities.*;
import app.pojo.ResponseGraphDetails;
import app.pojo.StudentGraphDetails;
import app.repositories.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.*;

@Component
public class GraphDataService {

    @Autowired
    private ClassEntityRepository classEntityRepository;

    @Autowired
    private LecturerClassesEntityRepository lecClassRepo;

    @Autowired
    private StudentClassesEntityRepository studentClassRepo;


    @Autowired
    private AnswerEntityRepository answerRepo;

    @Autowired
    private MinutePaperEntityRepository ompRepo;

    public List<ResponseGraphDetails> getResponseGraphData(String username) {
        List<ResponseGraphDetails> results = new ArrayList<>();
        if(username == null || username.isEmpty()) {
            return results;
        }

        List<ClassEntity> classes = new ArrayList<>();
        Optional<List<LecturerClassessEntity>> lecClassEntities = lecClassRepo.findAllByUsername(username);
        lecClassEntities.ifPresent(e -> {
            e.forEach(lecClass -> {
                classEntityRepository.findByClassCode(lecClass.getClassCode()).ifPresent(classes::add);
            });
        });

        classes.forEach(c -> {
            int numberOfStudents = studentClassRepo.countAllByClassCode(c.getClassCode());
            int numberOfPapers = ompRepo.countAllByClassCode(c.getClassCode());
            int numberOfResponses = getNumberOfResponses(c.getClassCode());
            ResponseGraphDetails rgd = new ResponseGraphDetails();
            rgd.setClassCode(c.getClassCode());
            rgd.setNumberSubmitted(numberOfResponses);
            rgd.setNumberPending((numberOfStudents * numberOfPapers) - numberOfResponses);
            rgd.setYear(c.getYear());
            results.add(rgd);
        });
        return results;
    }

    public List<StudentGraphDetails> getStudentGraphData(String classCode) {
        List<StudentGraphDetails> results = new ArrayList<>();
        if (classCode == null || classCode.isEmpty()) {
            return results;
        }
        Optional<List<StudentClassesEntity>> students = studentClassRepo.findAllByClassCode(classCode);

        students.ifPresent(elems-> {
            elems.forEach(student -> {
                results.add(new StudentGraphDetails(student.username, getNumberOfStudemtResponses(student.username, classCode)));
            });
        });

        return results;
    }

    private int getNumberOfResponses(String classCode) {
        int result = 0;
        List<MinutePaperEntity> papers = ompRepo.findAllByClassCodeAndTemplate(classCode,false).orElseGet(ArrayList::new);
        Set<String> responses = new HashSet<>();

        for (MinutePaperEntity paper : papers) {
            List<AnswerEntity> answerEntities = answerRepo.findAllByPaperIdAndSubmitted(paper.getPaperId(), true).orElseGet(ArrayList::new);
            for (AnswerEntity answer : answerEntities) {
                responses.add(answer.getUsername());
            }
            result += responses.size();
        }
        return result;
    }

    private int getNumberOfStudemtResponses(String username, String classCode) {
        Set<Integer> responses = new HashSet<>();
        List<MinutePaperEntity> papers = ompRepo.findAllByClassCodeAndTemplate(classCode,false).orElseGet(ArrayList::new);

        for (MinutePaperEntity paper : papers) {
            if (answerRepo.countAllByUsernameAndPaperIdAndSubmitted(username,paper.getPaperId(), true) > 0) {
                responses.add(paper.getPaperId());
            }
        }
        return responses.size();
    }

}
