package app.services;

import app.entities.AnswerEntity;
import app.entities.QuestionEntity;
import app.pojo.AnswerSet;
import app.repositories.AnswerEntityRepository;
import app.repositories.MinutePaperEntityRepository;
import app.repositories.QuestionEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.*;

@Component
public class MinutePaperService {

    @Autowired
    private MinutePaperEntityRepository minutePaperEntityRepository;

    @Autowired
    private QuestionEntityRepository questionEntityRepository;

    @Autowired
    AnswerEntityRepository answerEntityRepository;

    public List<QuestionEntity> getQuestions(String paperId) {
        if (paperId == null || paperId.isEmpty()) {
            return new ArrayList<>();
        }
        Optional<List<QuestionEntity>> questionEntities = questionEntityRepository.findAllByPaperIdOrderByQuestionIdAsc(UtilsService.parseId(paperId));
        return questionEntities.orElseGet(ArrayList::new);
    }

    public List<AnswerSet> getAnswerSets(String paperId) {
        if (paperId == null || paperId.isEmpty()) {
            return new ArrayList<>();
        }

        List<AnswerEntity> answers = getAllAnswers(paperId);
        Set<String> usernames = new HashSet<>();
        List<AnswerSet> answerSets = new ArrayList<>();

        answers.forEach(answer -> {
            usernames.add(answer.getUsername());
        });

        if(usernames.size() > 0) {
            usernames.forEach(username -> {
                answerSets.add(new AnswerSet(new ArrayList<>()));

                answers.forEach(answer -> {
                    if (answer.getUsername().equals(username)) {
                        if(isAnonymous(UtilsService.parseId(paperId))) {
                            answer.setUsername("Anonymous");
                        }
                        answerSets.get(username.indexOf(username)).getAnswerSets().add(answer);
                    }
                });
            });
            return answerSets;
        }
        return new ArrayList<>();
    }

    private List<AnswerEntity> getAllAnswers(String paperId) {
        if (paperId == null || paperId.isEmpty()) {
            return new ArrayList<>();
        }

        Optional<List<AnswerEntity>> answerEntities = answerEntityRepository.findAllByPaperIdAndSubmitted(UtilsService.parseId(paperId),true);
        return answerEntities.orElseGet(ArrayList::new);
    }

    private Boolean isAnonymous(int paperId) {
        return minutePaperEntityRepository.findByPaperIdAndAnonymous(paperId, true).isPresent();
    }
}
