package app.services;

import app.analysis.sentiment.SentimentAnalyzer;
import app.analysis.sentiment.SentimentResult;
import app.entities.AnswerEntity;
import app.pojo.WordDetails;
import app.repositories.AnswerEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ResourceLoader;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
public class SentimentService {

    @Autowired
    private AnswerEntityRepository answerRepo;


    public List<WordDetails> generateWordCloud(String paperId) {
        List<WordDetails> result = new ArrayList<>();
        List<AnswerEntity> answers = answerRepo.findAllByPaperIdAndSubmitted(UtilsService.parseId(paperId), true).orElseGet(ArrayList::new);
        if(answers.size() > 0) {
            List<String> stopwords = getStopwords();
            SentimentAnalyzer analyzer = new SentimentAnalyzer();
            Map<AnswerEntity, SentimentResult> sentimentResultMap = new HashMap<>();
            Map<String, WordDetails> wordcloud = new HashMap<>();

            answers.forEach(answerEntity -> sentimentResultMap.put(answerEntity, analyzer.analyse(answerEntity.getAnswerText())));
            sentimentResultMap.forEach((k,v) -> {
                List<String> filteredText = v.getText().stream().filter(word -> !stopwords.contains(word.toLowerCase())).collect(Collectors.toList());
                v.setText(filteredText);
            });

            sentimentResultMap.forEach((k,v) -> {
                v.getText().forEach(word -> {
                    WordDetails details;
                    if(wordcloud.get(word) == null) {
                        details = new WordDetails();
                        details.setWord(word);
                        details.setPolarity(v.getScore());
                        details.setNumberOfOccurrences(1);
                        details.setQuestionId(k.getQuestionId());
                        wordcloud.put(word, details);
                    } else {
                        details = wordcloud.get(word);
                        details.setPolarity(calculateScore(details, v.getScore()));
                        details.setNumberOfOccurrences(details.getNumberOfOccurrences() + 1);
                    }
                });
            });
            wordcloud.forEach((k,v) -> result.add(v));
        }
        return result;
    }


    private int calculateScore(WordDetails currentWord, int score) {
        if(score == currentWord.getPolarity()) {
            return score;
        }

        if (score < currentWord.getPolarity()) {
            return score + 1;
        } else {
            return score - 1;
        }
    }

    private List<String> getStopwords() {
        List<String> stopwords = new ArrayList<>();
        ClassLoader loader = getClass().getClassLoader();
        File file;

        try {
            file = new File(loader.getResource("stopwords.txt").getFile());
        } catch (NullPointerException npe) {
            return new ArrayList<>();
        }

        try (BufferedReader reader = new BufferedReader(new FileReader(file))){
            String line;
            while ((line = reader.readLine()) != null) {
                stopwords.add(line);
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
            return new ArrayList<>();
        }
        return stopwords;
    }

}
