package app.services;

import app.entities.AnswerEntity;
import app.entities.ClassEntity;
import app.entities.MinutePaperEntity;
import app.pojo.ResponseDetailsWrapper;
import app.repositories.AnswerEntityRepository;
import app.repositories.MinutePaperEntityRepository;
import app.repositories.QuestionEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Component
public class StudentInfoService {

    @Autowired
    private MinutePaperEntityRepository minutePaperEntityRepository;

    @Autowired
    private AnswerEntityRepository answerEntityRepository;

    @Autowired
    private QuestionEntityRepository questionEntityRepository;

    public List<MinutePaperEntity> getOpenPapers(String username, List<ClassEntity> classes) {
        if(username == null || username.isEmpty() || classes.size() <= 0) {
            return new ArrayList<>();
        }
        List<MinutePaperEntity> result = new ArrayList<>();
        List<MinutePaperEntity> papersForAllClasses = new ArrayList<>();
        Set<Integer> paperIdSet = new HashSet<>();

        Optional<List<AnswerEntity>> responses = answerEntityRepository.findAllByUsernameAndSubmitted(username,false);
        responses.ifPresent(response -> response.forEach( e -> {
            paperIdSet.add(e.getPaperId());
        }));

        classes.forEach(c -> {
            minutePaperEntityRepository.findAllByClassCodeAndTemplate(c.getClassCode(), false).ifPresent(papersForAllClasses::addAll);
        });

        papersForAllClasses.forEach(paper -> {
            if(!answerEntityRepository.existsByUsernameAndPaperId(username, paper.getPaperId())) {
                paperIdSet.add(paper.getPaperId());
            }
        });

        paperIdSet.forEach(id -> {
            minutePaperEntityRepository.findByPaperId(id).ifPresent(result::add);
        });

        return result;
    }

    public List<MinutePaperEntity> getClosedClassPapers(String username, String classCode) {
        if((username == null || username.isEmpty()) || (classCode == null || classCode.isEmpty())) {
            return new ArrayList<>();
        }

        List<MinutePaperEntity> result = new ArrayList<>();
        Optional<List<MinutePaperEntity>> possiblePapers = minutePaperEntityRepository.findAllByClassCodeAndTemplate(classCode,false);
        Optional<List<AnswerEntity>> possibleSubmissions = answerEntityRepository.findAllByUsernameAndSubmitted(username,true);
        Set<Integer> paperIds = new LinkedHashSet<>();

        possibleSubmissions.ifPresent(submissions -> submissions.forEach(e -> {paperIds.add(e.getPaperId());}));

        possiblePapers.ifPresent(papers -> papers.forEach(e -> {
            if(paperIds.contains(e.getPaperId())) {
                result.add(e);
            }
        }));

        return result;
    }

    public List<AnswerEntity> getStudentAnswers(String username, String paperId) {
        if ((username == null || username.isEmpty()) || (paperId == null || paperId.isEmpty())) {
            return new ArrayList<>();
        }
        Optional<List<AnswerEntity>> answerEntities = answerEntityRepository.findAllByUsernameAndPaperIdOrderByAnswerIdAsc(username,UtilsService.parseId(paperId));

        return answerEntities.orElseGet(ArrayList::new);
    }

    public void save(ResponseDetailsWrapper answers, Boolean submitted) {
        if(answers.getResponses().size() > 0) {
            try {
                answers.getResponses().forEach(response -> {
                    AnswerEntity entity = new AnswerEntity();
                    int existing = hasExistingAnswer(response.getUsername(),UtilsService.parseId(response.getPaperId()),UtilsService.parseId(response.getQuestionId()));

                    if( existing != -1) {
                        entity.setAnswerId(existing);
                    }

                    entity.setAnswerText(cleanApostrpohes(response.getAnswerText()));
                    entity.setUsername(response.getUsername());
                    entity.setSubmitted(submitted);
                    entity.setPaperId(UtilsService.parseId(response.getPaperId()));
                    entity.setQuestionId(UtilsService.parseId(response.getQuestionId()));
                    entity.setSubmissionDate(new Date());
                    answerEntityRepository.save(entity);
                });
            } catch (Exception e) {
                //Tear down
            }
        }
    }

    private String cleanApostrpohes(String text) {
        Pattern p = Pattern.compile("'");
        Matcher m = p.matcher(text);
        StringBuffer sb = new StringBuffer();
        while(m.find()) {
            m.appendReplacement(sb,"");
        }
        m.appendTail(sb);
        return sb.toString();
    }

    private int hasExistingAnswer(String username, int paperId, int questionId) {
        if(answerEntityRepository.existsByUsernameAndPaperIdAndQuestionId(username,paperId,questionId)) {
            AnswerEntity entity = answerEntityRepository.findByUsernameAndPaperIdAndQuestionId(username,paperId,questionId).get();
            if(entity.getSubmitted()) {
                return -1;
            }
            return entity.getAnswerId();
        }
        return -1;
    }
}
